@@ -1,28 +1,20 @@
-# coding=utf-8
-
-from __future__ import unicode_literals
-
-from contextlib import contextmanager
 from unittest import TestCase
 
 from .sanitizer import Sanitizer
 
 
+default_sanitizer = Sanitizer()
+
+
 class SanitizerTestCase(TestCase):
-    if not hasattr(TestCase, "subTest"):
-
-        @contextmanager
-        def subTest(self, *args, **kwargs):
-            yield
-
-    def run_tests(self, entries, sanitizer=Sanitizer()):
+    def run_tests(self, entries, *, sanitizer=default_sanitizer, strip=False):
         for before, after in entries:
             with self.subTest(before=before, after=after):
                 after = before if after is None else after
                 result = sanitizer.sanitize(before)
                 self.assertEqual(
-                    result,
-                    after,
+                    result.strip() if strip else result,
+                    after.strip() if strip else after,
                     "Cleaning '%s', expected '%s' but got '%s'"
                     % (
                         before.encode("unicode-escape"),
@@ -165,6 +157,7 @@ class SanitizerTestCase(TestCase):
         entries = (
             ("<li> - foo</li>", "<li>foo</li>"),
             ("<li>* foo</li>", "<li>foo</li>"),
+            ("<li>*foo</li>", "<li>*foo</li>"),
         )
 
         self.run_tests(entries)
@@ -450,9 +443,18 @@ Mitarbeitenden folgende geschäftlich bedingten Auslagen ersetzt:</font></p>
     def test_anchor_names(self):
         self.run_tests(
             [
-                ('<a name="test" id="test"></a>', '<a name="test"></a>',),
-                ('<a name="test"></a>', '<a name="test"></a>',),
-                ('<a id="test"></a>', '<a name="test"></a>',),
+                (
+                    '<a name="test" id="test"></a>',
+                    '<a name="test"></a>',
+                ),
+                (
+                    '<a name="test"></a>',
+                    '<a name="test"></a>',
+                ),
+                (
+                    '<a id="test"></a>',
+                    '<a name="test"></a>',
+                ),
             ],
         )
 
@@ -510,3 +512,60 @@ Mitarbeitenden folgende geschäftlich bedingten Auslagen ersetzt:</font></p>
                 }
             ),
         )
+
+    def test_billion_laughs(self):
+        source = """\
+<?xml version="1.0"?>
+<!DOCTYPE lolz [
+ <!ENTITY lol "lol">
+ <!ELEMENT lolz (#PCDATA)>
+ <!ENTITY lol1 "&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;&lol;">
+ <!ENTITY lol2 "&lol1;&lol1;&lol1;&lol1;&lol1;&lol1;&lol1;&lol1;&lol1;&lol1;">
+ <!ENTITY lol3 "&lol2;&lol2;&lol2;&lol2;&lol2;&lol2;&lol2;&lol2;&lol2;&lol2;">
+ <!ENTITY lol4 "&lol3;&lol3;&lol3;&lol3;&lol3;&lol3;&lol3;&lol3;&lol3;&lol3;">
+ <!ENTITY lol5 "&lol4;&lol4;&lol4;&lol4;&lol4;&lol4;&lol4;&lol4;&lol4;&lol4;">
+ <!ENTITY lol6 "&lol5;&lol5;&lol5;&lol5;&lol5;&lol5;&lol5;&lol5;&lol5;&lol5;">
+ <!ENTITY lol7 "&lol6;&lol6;&lol6;&lol6;&lol6;&lol6;&lol6;&lol6;&lol6;&lol6;">
+ <!ENTITY lol8 "&lol7;&lol7;&lol7;&lol7;&lol7;&lol7;&lol7;&lol7;&lol7;&lol7;">
+ <!ENTITY lol9 "&lol8;&lol8;&lol8;&lol8;&lol8;&lol8;&lol8;&lol8;&lol8;&lol8;">
+]>
+<lolz>&lol9;</lolz>
+"""
+
+        external_entities = """\
+ <?xml version="1.0"?>
+  <!DOCTYPE foo [
+   <!ELEMENT foo ANY >
+   <!ENTITY xxe SYSTEM "file:///dev/random" >]><foo>&xxe;</foo>
+"""
+
+        self.run_tests(
+            [
+                (source, "             ]&gt; &amp;lol9; "),
+                (external_entities, "    ]&gt;&amp;xxe; "),
+            ],
+            strip=True,
+        )
+
+    def test_data_attributes(self):
+        sanitizer = Sanitizer(
+            {
+                "tags": ["span"],
+                "empty": (),
+                "separate": (),
+                "attributes": {"span": {"data-title"}},
+            }
+        )
+
+        entries = (
+            (
+                '<span data-title="Test">Content</span>',
+                '<span data-title="Test">Content</span>',
+            ),
+            (
+                '<span data-title="Test" data-other="Stuff">Content</span>',
+                '<span data-title="Test">Content</span>',
+            ),
+        )
+
+        self.run_tests(entries, sanitizer=sanitizer)
